! =================================================================================================================================
! MODULE       : stomate_growth_res_lim
!
! CONTACT      : orchidee-help _at_ ipsl.jussieu.fr
!
! LICENCE      : IPSL (2006)
!                This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF       Plant growth and C-allocation among the biomass components (leaves, wood, roots, fruit, reserves) 
!!               is calculated making use of the resource limitation scheme proposed by Friedlingstein et al 1999.      
!!
!!\n DESCRIPTION: This module calculates three processes: (1) daily maintenance respiration based on the half-hourly
!!               respiration calculated in stomate_resp.f90, (2) the allocation fractions to the different biomass
!!               components based on the resource-limitation approach and (3) the allocatable biomass as the residual
!!               of GPP-Ra. Multiplication of the allocation fractions and allocatable biomass given the changes in
!!               biomass pools.
!!
!! RECENT CHANGE(S): Until 1.9.6 the calculations in this routine were distributed over stomate_alloc.f90 and
!!               stomate_npp.f90. Given the strong dependencies of both routines they were merged in this module.
!!               At the same time an alternative growth module (stomate_growth_fun_all.f90) was introduced.               
!!
!! REFERENCE(S)	: - Friedlingstein, P., G. Joel, C.B. Field, and Y. Fung (1999), Towards an allocation
!!               scheme for global terrestrial carbon models, Global Change Biology, 5, 755-770.\n
!!
!! SVN          :
!! $HeadURL$
!! $Date$
!! $Revision$
!! \n
!_ ================================================================================================================================

MODULE stomate_growth_res_lim

  ! Modules used:

  USE ioipsl_para
  USE pft_parameters
  USE stomate_data
  USE constantes
  USE constantes_soil

  IMPLICIT NONE

  ! Private & public routines

  PRIVATE
  PUBLIC growth_res_lim_clear, growth_res_lim

  ! Variables shared by all subroutines in this module

  LOGICAL, SAVE                                             :: firstcall = .TRUE.  !! Is this the first call? (true/false) 
  
  CONTAINS


!! ================================================================================================================================
!! SUBROUTINE   : growth_res_lim_clear
!!
!>\BRIEF          Set the flag ::firstcall to .TRUE. and as such activate section 
!! 1.1 of the subroutine alloc (see below).\n
!!
!_ ================================================================================================================================

  SUBROUTINE growth_res_lim_clear
    firstcall = .TRUE.
  END SUBROUTINE growth_res_lim_clear


!! ================================================================================================================================
!! SUBROUTINE 	: growth_res_lim
!!
!>\BRIEF        Calculate NPP as the difference between GPP and respiration (= growth + 
!! maintenance respiration). Distribute NPP over all compartments (carbon reserves, 
!! aboveground sapwood, belowground sapwood, root, fruits and leaves following 
!! Friedlingstein et al. (1999).
!!
!! DESCRIPTION  : NPP is calculated from three components: Gross Primary Productivity 
!! (GPP), maintenance respiration and growth respiration 
!! (all in @tex $ gC.m^{-2}dt^{-1} $ @endtex), following the convention that positive 
!! fluxes denote fluxes from the plants to the atmosphere. GPP is the input variable from
!! which, in the end, NPP or total allocatable biomass @tex $(gC.m^{-2})$ @endtex is 
!! calculated. Net primary production is then calculated as:\n	
!! NPP = GPP - growth_resp - maint-resp   [eq. 1]\n   
!!	
!! The calculation of maintenance respiration is done in routine stomate_resp.f90. 
!! Maintenance respiration is calculated for the whole plant and is therefore removed from 
!! the total allocatable biomass. In order to prevent all allocatable biomass from being 
!! used for maintenance respiration, a limit fraction of total allocatable biomass, tax_max, 
!! is defined (in the variables declaration). If maintenance respiration exceeds tax_max 
!! (::bm_tax_max), the maximum allowed allocatable biomass will be respired and the remaining 
!! respiration, required in excess of tax_max, is taken out from tissues already present in 
!! the plant biomass.\n  
!! 
!! After total allocatable biomass has been updated by removing maintenance respiration, 
!! total allocatable biomass is distributed to all plant compartments according to the 
!! f_alloc fractions also calculated in this routine.\n
!! 
!! The philosophy underlying the allocation scheme is that allocation patterns
!! result from evolved responses that adjust carbon investments to facilitate capture of 
!! most limiting resources i.e. light, water and mineral nitrogen. The implemented scheme 
!! calculates the limitation of light, water and nitrogen. However, nitrogen is not a 
!! prognostic variable of the model and therefore soil temperature and soil moisture 
!! are used as a proxy for soil nitrogen availability.\n
!! Sharpe & Rykiel (1991) proposed a generic relationship between the allocation of 
!! carbon to a given plant compartment and the availability of a particular resource:\n
!! \latexonly 
!!   \input{alloc1.tex}
!! \endlatexonly
!! \n
!! where A is the allocation of biomass production (NPP) to a given compartment (either 
!! leaves, stem, or roots). Xi and Yj are resource availabilities (e.g. light, water, 
!! nutrient). For a given plant compartment, a resource can be of type X or Y. An increase 
!! in a X-type resource will increase the allocation to compartment A. An increase in a 
!! Y-type resource will, however, lead to a decrease in carbon allocation to that compartment. 
!! In other words, Y-type resources are those for which uptake increases with increased 
!! investment in the compartment in question. X-type resources, as a consequence of 
!! trade-offs, are the opposite. For example, water is a Y-type resource for root allocation. 
!! Water-limited conditions should promote carbon allocation to roots, which enhance water 
!! uptake and hence minimize plant water stress. Negative relationships between investment 
!! and uptake arise when increased investment in one compartment leads, as required for 
!! conservation of mass, to decreased investment in a component involved in uptake of 
!! that resource.\n
!!
!! The implemented scheme allocates carbon to the following components:\n
!! - Carbon reserves;\n
!! - Aboveground sapwood;\n
!! - Belowground sapwood;\n
!! - Roots;\n
!! - Fruits/seeds and\n
!! - Leaves.
!! \n
!!
!! The allocation to fruits and seeds is simply a 10% "tax" of the total biomass 
!! production.\n
!! Following carbohydrate use to support budburst and initial growth, the 
!! carbohydrate reserve is refilled. The daily amount of carbon allocated to the 
!! reserve pool is proportional to leaf+root allocation (::LtoLSR and ::RtoLSR).
!! Sapwood and root allocation (respectively ::StoLSR and ::RtoLSR) are proportional 
!! to the estimated light and soil (water and nitrogen) stress (::Limit_L and 
!! ::Limit_NtoW). Further, Sapwood allocation is separated in belowground sapwood 
!! and aboveground sapwood making use of the parameter (:: alloc_sap_above_tree
!! or ::alloc_sap_above_grass). For trees partitioning between above and 
!! belowground compartments is a function of PFT age. Leaf allocation (::LtoLSR) 
!! is calculated as the residual of root and sapwood 
!! allocation (LtoLSR(:) = 1. - RtoLSR(:) - StoLSR(:).\n
!!
!! Growth respiration is calculated as a fraction of allocatable biomass for each part 
!! of the plant. The fraction coefficient ::frac_growth_resp is defined in 
!! stomate_constants.f90 and is currently set to be the same for all plant compartments. 
!! Is it thus assumed that it takes as much energy to grow a leaf as it is to grow 
!! wood. Allocatable biomass of all plant compartments are updated by removing what is 
!! lost through growth respiration. Net allocatable biomass (total allocatable biomass 
!! after maintenance and growth respiration) is added to the current biomass for each 
!! plant compartment.\n
!!
!! Finally, leaf age and plant age are updated. Leaf age is described with the concept 
!! of "leaf age classes". A number of leaf age classes (::nleafages) is defined in 
!! stomate_constants.f90. Each leaf age class contains a fraction (::leaf_frac) of the 
!! total leaf biomass. When new biomass is added to leaves, the age of the biomass in 
!! the youngest leaf age class is decreased. The fractions of leaves in the other leaf 
!! ages classes are also updated as the total biomass has increased. Plant age is 
!! updated first by increasing the age of the previous biomass by one time step, and 
!! then by adjusting this age as the average of the ages of the previous and the new 
!! biomass.
!!
!! RECENT CHANGE(S): Until 1.9.6 the calculations in this routine were distributed over 
!! stomate_alloc.f90 and stomate_npp.f90. Given the strong dependencies of both routines 
!! they were merged in a single module. The underlying science and principles were not 
!! changed. This new module (stomate_growth_res_lim) exactely reproduces the results 
!! from the previous implementation (stomate_alloc and stomate_npp). At the same time an 
!! alternative growth module (stomate_growth_fun_all.f90), based on the pipe-model, was 
!! added to the code.
!!
!! MAIN OUTPUT VARIABLE(S): ::npp and ::biomass; fraction of NPP that is allocated to the 
!! six different biomass compartments (leaves, roots, above and belowground wood, 
!! carbohydrate reserves and fruits). DIMENSION(npts,nvm,nparts).
!!
!! REFERENCE(S)	:
!! - Friedlingstein, P., G. Joel, C.B. Field, and Y. Fung (1999), Towards an allocation
!! scheme for global terrestrial carbon models, Global Change Biology, 5, 755-770.\n
!! - Krinner G, Viovy N, de Noblet-Ducoudr N, Ogee J, Polcher J, Friedlingstein P,
!! Ciais P, Sitch S, Prentice I C (2005) A dynamic global vegetation model for studies
!! of the coupled atmosphere-biosphere system. Global Biogeochemical Cycles, 19, GB1015,
!! doi: 10.1029/2003GB002199.\n
!! - F.W.T.Penning De Vries, A.H.M. Brunsting, H.H. Van Laar. 1974. Products, requirements 
!! and efficiency of biosynthesis a quantitative approach. Journal of Theoretical Biology, 
!! Volume 45, Issue 2, June 1974, Pages 339-377.
!! - Sharpe, P.J.H., and Rykiel, E.J. (1991), Modelling integrated response of plants 
!! to multiple stresses. In: Response of Plants to Multiple Stresses (eds Mooney, H.A., 
!! Winner, W.E., Pell, E.J.), pp. 205-224, Academic Press, San Diego, CA.\n
!!
!! +++++++++++++++++++++
!! MAKE A NEW FLOW CHART
!! +++++++++++++++++++++
!! FLOWCHART    : 
!! \latexonly
!! \includegraphics[scale=0.14]{stomate_npp_flow.jpg}
!! \endlatexonly
!! \n 
!! \latexonly 
!!   \includegraphics[scale=0.5]{allocflow.jpg}
!! \endlatexonly
!! \n
!_ ================================================================================================================================
  
  SUBROUTINE growth_res_lim (npts, dt, lai, veget_max, &
       PFTpresent, senescence, when_growthinit, moiavail_week, &
       soilhum_month, tsoil_month, gpp, resp_maint_part, &
       resp_maint, resp_growth, npp, biomass, &
       age, leaf_age, leaf_frac)
!, use_reserve)                                              
 

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                               :: npts			!! Domain size - number of grid cells 
                                                                                        !! (unitless)
    REAL(r_std), INTENT(in)                                  :: dt			!! Time step of the simulations for stomate
                                                                                        !! (days)    
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)             :: lai			!! PFT leaf area index 
                                                                                        !! @tex $(m^2 m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)             :: veget_max		!! PFT "Maximal" coverage fraction of a PFT  
                                                                                        !! @tex $(m^2 m^{-2})$ @endtex   
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)             :: when_growthinit	        !! Days since beginning of growing season 
                                                                                        !! (days)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)             :: moiavail_week		!! PFT moisture availability - integrated
                                                                                        !! over a week (0-1, unitless) 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)             :: gpp                     !! PFT gross primary productivity 
                                                                                        !! @tex $(gC.m^{-2}dt^{-1})$ @endtex                                       
    REAL(r_std), DIMENSION(npts,nbdl), INTENT(in)            :: tsoil_month		!! PFT soil temperature - integrated over 
                                                                                        !! a month (K) 
    REAL(r_std), DIMENSION(npts,nbdl), INTENT(in)            :: soilhum_month		!! PFT soil humidity - integrated over a 
                                                                                        !! month (0-1, unitless)
    REAL(r_std), DIMENSION(npts,nvm,nparts), INTENT(in)      :: resp_maint_part         !! Maintenance respiration of different plant 
                                                                                        !! parts @tex $(gC.m^{-2}dt^{-1})$ @endtex  
    LOGICAL, DIMENSION(npts,nvm), INTENT(in)                 :: senescence		!! Is the PFT senescent?  - only for 
                                                                                        !! deciduous trees (true/false)
    LOGICAL, DIMENSION(npts,nvm), INTENT(in)                 :: PFTpresent              !! PFT exists (true/false)
    
    !! 0.2 Output variables

    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)            :: resp_maint              !! PFT maintenance respiration 
                                                                                        !! @tex $(gC.m^{-2}dt^{-1})$ @endtex		    
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)            :: resp_growth             !! PFT growth respiration 
                                                                                        !! @tex $(gC.m^{-2}dt^{-1})$ @endtex				
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)            :: npp                     !! PFT net primary productivity 
                                                                                        !! @tex $(gC.m^{-2}dt^{-1})$ @endtex

    !! 0.3 Modified variables

    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)          :: age                     !! PFT age (years)
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), INTENT(inout)   :: biomass	!! PFT total biomass 
                                                                                        !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nleafages), INTENT(inout):: leaf_age		!! PFT age of different leaf classes 
                                                                                        !! (days)
    REAL(r_std), DIMENSION(npts,nvm,nleafages), INTENT(inout):: leaf_frac		!! PFT fraction of leaves in leaf age class
                                                                                        !! (0-1, unitless)

    !! 0.4 Local variables
                          
    INTEGER(i_std)                                           :: i,j,k,l,m	        !! Indices (unitless)
    REAL(r_std)                                              :: reserve_time		!! Maximum number of days during which 
                                                                                        !! carbohydrate reserve may be used (days)
    REAL(r_std), DIMENSION(nvm)                              :: lai_happy_old		!! Lai threshold below which carbohydrate 
                                                                                        !! reserve may be used 
                                                                                        !! @tex $(m^2 m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts)                             :: limit_L	        	!! Lights stress (0-1, unitless)
    REAL(r_std), DIMENSION(npts)                             :: limit_N	        	!! Total nitrogen stress (0-1, unitless)
    REAL(r_std), DIMENSION(npts)                             :: limit_N_temp		!! Stress from soil temperature on nitrogen
                                                                                        !! mineralisation (0-1, unitless)
    REAL(r_std), DIMENSION(npts)                             :: limit_N_hum		!! Stress from soil humidity on nitrogen 
                                                                                        !! mineralisation (0-1, unitless)
    REAL(r_std), DIMENSION(npts)                             :: limit_W		        !! Soil water stress (0-1, unitless)
    REAL(r_std), DIMENSION(npts)                             :: limit_WorN		!! Most limiting factor in the soil: 
                                                                                        !! nitrogen or water (0-1, unitless)
    REAL(r_std), DIMENSION(npts)                             :: limit			!! Most limiting factor: amongst limit_N, 
                                                                                        !! limit_W and limit_L (0-1, unitless)
    REAL(r_std), DIMENSION(npts)                             :: t_nitrogen		!! Preliminairy soil temperature stress 
                                                                                        !! used as a proxy for nitrogen stress (K)
    REAL(r_std), DIMENSION(npts)                             :: h_nitrogen		!! Preliminairy soil humidity stress used 
                                                                                        !! as a proxy for nitrogen stress 
                                                                                        !! (unitless)  
    REAL(r_std), DIMENSION(npts)                             :: rpc			!! Scaling factor for integrating vertical
                                                                                        !! soil profiles (unitless) 	
    REAL(r_std), DIMENSION(npts)                             :: LtoLSR		        !! Ratio between leaf-allocation and 
                                                                                        !! (leaf+sapwood+root)-allocation 
                                                                                        !! (0-1, unitless)
    REAL(r_std), DIMENSION(npts)                             :: StoLSR		        !! Ratio between sapwood-allocation and 
                                                                                        !! (leaf+sapwood+root)-allocation 
                                                                                        !! (0-1, unitless)
    REAL(r_std), DIMENSION(npts)                             :: RtoLSR		        !! Ratio between root-allocation and 
                                                                                        !! (leaf+sapwood+root)-allocation 
                                                                                        !! (0-1, unitless)
    REAL(r_std), DIMENSION(npts)                             :: carb_rescale		!! Rescaling factor for allocation factors
                                                                                        !! if carbon is allocated to carbohydrate 
                                                                                        !! reserve (0-1, unitless)
    REAL(r_std), DIMENSION(npts)                             :: natveg_tot		!! Total natural vegetation cover on 
                                                                                        !! natural part of the grid cell 
                                                                                        !! (0-1, unitless)
    REAL(r_std), DIMENSION(npts)                             :: lai_nat		        !! Average LAI on natural part of the grid
                                                                                        !! cell @tex $(m^2 m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts)                             :: alloc_sap_above	        !! Fraction of sapwood 
                                                                                        !! allocation to above ground sapwood 
                                                                                        !! (0-1, unitless)
    REAL(r_std), DIMENSION(npts)                             :: bm_add                  !! Biomass increase @tex $(gC.m^{-2})$ @endtex		
    REAL(r_std), DIMENSION(npts)                             :: bm_new                  !! New biomass @tex $(gC.m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts)                             :: bm_tax_max              !! Maximum part of allocatable biomass used for 
                                                                                        !! respiration @tex $(gC.m^{-2})$ @endtex	
    REAL(r_std), DIMENSION(npts)                             :: bm_pump                 !! Biomass that remains to be taken away 
                                                                                        !! @tex $(gC.m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm)                         :: transloc_leaf		!! Fraction of carbohydrate reserve used 
                                                                                        !! (::use_reserve) to support leaf growth 
                                                                                        !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm)                         :: use_reserve             !! Mass taken from carbohydrate reserve
                                                                                        !! @tex $(gC m^{-2})$ @endtex				
    REAL(r_std), DIMENSION(npts,nvm)                         :: bm_create               !! Biomass created when biomass<0 because of dark
                                                                                        !! respiration @tex $(gC.m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm)                         :: bm_alloc_tot            !! Allocatable biomass for the whole plant
                                                                                        !! @tex $(gC.m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm)                         :: dia                     !! Tree diameter (cm)
    REAL(r_std), DIMENSION(npts,nvm)                         :: ltor                    !! Leaf to root ratio (unitless)
    REAL(r_std), DIMENSION(npts,nvm)                         :: leaf_mass_young	        !! Leaf biomass in youngest leaf age class 
                                                                                        !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm)                         :: lm_old		        !! Variable to store leaf biomass from 
                                                                                        !! previous time step 
                                                                                        !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm)                         :: lai_around		!! lai on natural part of the grid cell, or
                                                                                        !! of agricultural PFTs 
                                                                                        !! @tex $(m^2 m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm)                         :: veget_max_nat 	        !! Vegetation cover of natural PFTs on the 
                                                                                        !! grid cell (agriculture masked) 
                                                                                        !! (0-1, unitless)
    REAL(r_std), DIMENSION(npts,nparts)                      :: resp_growth_part        !! Growth respiration of different plant parts
                                                                                        !! @tex $(gC.m^{-2}dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nparts)                  :: f_alloc		        !! PFT fraction of NPP that is allocated to
                                                                                        !! the different components (0-1, unitless)
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements)        :: bm_alloc                !! PFT biomass increase, i.e. NPP per plant part 
                                                                                        !! @tex $(gC.m^{-2}dt^{-1})$ @endtex**2
    REAL(r_std), SAVE, DIMENSION(0:nbdl)                     :: z_soil		        !! Variable to store depth of the different
                                                                                        !! soil layers (m)
!_ =================================================================================================================================


!! 1. Initialize

    IF (bavard.GE.3) WRITE(numout,*) 'Entering resource limited growth'

    !! 1.1 First call only
    IF ( firstcall ) THEN
      
       !! 1.1.1 Initialization
       L0(2:nvm) = un - R0 - S0(2:nvm)
       IF ((MINVAL(L0(2:nvm)) .LT. zero) .OR. (MAXVAL(S0(2:nvm)) .EQ. un)) THEN 

          CALL ipslerr_p (3,'in module stomate_alloc', &
               &           'Something wrong happened', &
               &           'L0 negative or division by zero if S0 = 1', &
               &           '(Check your parameters.)')

       ENDIF


       !! 1.1.2 Copy the depth of the different soil layers (number of layers=nbdl) 
       !        previously calculated as variable diaglev in routines sechiba.f90 and slowproc.f90
       z_soil(0) = 0.
       z_soil(1:nbdl) = diaglev(1:nbdl)

       !! 1.1.3 Print flags and parameter settings
       WRITE(numout,*) 'allocation:'
       WRITE(numout,'(a)',ADVANCE='NO') '    > We'
       !IF ( .NOT. ok_minres ) WRITE(numout,'(a,$)') ' do NOT'
       WRITE(numout,*) 'try to reach a minumum reservoir when severely stressed.'
       WRITE(numout,*) '   > Time delay (days) to build leaf mass (::tau_leafinit): ', tau_leafinit
       WRITE(numout,*) '   > Curvature of root mass with increasing soil depth (::z_nitrogen): ', z_nitrogen
       WRITE(numout,*) '   > Sap allocation above the ground / total sap allocation (0-1, unitless): '
       WRITE(numout,*) '       grasses (::alloc_sap_above_grass) :', alloc_sap_above_grass
       WRITE(numout,*) '   > Default root alloc fraction (1; ::R0): ', R0
       WRITE(numout,*) '   > Default sapwood alloc fraction (1; ::S0): ', S0(:)
       WRITE(numout,*) '   > Default fruit allocation (1, ::f_fruit): ', f_fruit
       WRITE(numout,*) '   > Minimum (min_LtoLSR)/maximum (::max_LtoLSR)leaf alloc fraction (0-1, unitless): ',&
            min_LtoLSR,max_LtoLSR
       WRITE(numout,*) '   > Maximum time (days) the carbon reserve can be used:'
       WRITE(numout,*) '       trees (reserve_time_tree):',reserve_time_tree
       WRITE(numout,*) '       grasses (reserve_time_grass):',reserve_time_grass
  
       firstcall = .FALSE.

    ENDIF
    
    !! 1.2 Every call

    !! 1.2.1 Reset output variable (::f_alloc)
    f_alloc(:,:,:) = zero
    f_alloc(:,:,icarbres) = un
    bm_alloc(:,:,:,icarbon) = zero
    resp_maint(:,:) = zero
    resp_growth(:,:) = zero
    npp(:,:) = zero
    use_reserve(:,:) = zero
 

    !! 1.2.2 Proxy for soil nitrogen stress
    !  Nitrogen availability and thus N-stress can not be calculated by the model. Water and
    !  temperature stress are used as proxy under the assumption that microbial activity is 
    !  determined by soil temperature and water availability. In turn, microbial activity is 
    !  assumed to be an indicator for nitrogen mineralisation and thus its availability. 

    !! 1.2.2.1 Convolution of nitrogen stress with root profile 
    !  Here we calculate preliminary soil temperature and soil humidity stresses that will be used 
    !  as proxies for nitrogen stress. Their calculation follows the nitrogen-uptake capacity of roots.
    !  The capacity of roots to take up nitrogen is assumed to decrease exponentially with 
    !  increasing soil depth. The curvature of the exponential function describing the 
    !  nitrogen-uptake capacity of roots (= root mass * uptake capacity) is given by 
    !  ::z_nitrogen. Strictly speaking its unit is meters (m). Despite its units this parameter 
    !  has no physical meaning.
    !  Because the roots are described by an exponential function but the soil depth is limited to
    !  ::z_soil(nbdl), the root profile is truncated at ::z_soil(nbdl). For numerical reasons, 
    !  the total capacity of the soil profile for nitrogen uptake should be 1. To this aim a scaling
    !  factor (::rpc) is calculated as follows:\n
    !  \latexonly
    !  \input{alloc2.tex} 
    !  \endlatexonly
    !  Then temperature (::t_nitrogen) and humidity (::h_nitrogen) proxies for nitrogen stress are 
    !  calculated using mean weighted (weighted by nitrogen uptake capacity) soil temperature (::tsoil_month)
    !  or soil moisture (::soil_hum_month) (calculated in stomate_season.f90). 
    !  \latexonly
    !  \input{alloc3.tex} 
    !  \endlatexonly
    !  \latexonly
    !  \input{alloc4.tex} 
    !  \endlatexonl	
    !  \n 
    ! Scaling factor for integration
    rpc(:) = un / ( un - EXP( -z_soil(nbdl) / z_nitrogen ) )

    ! Integrate over # soil layers
    t_nitrogen(:) = zero

    DO l = 1, nbdl ! Loop over # soil layers

       t_nitrogen(:) = t_nitrogen(:) + tsoil_month(:,l) * rpc(:) * &
            ( EXP( -z_soil(l-1)/z_nitrogen ) - EXP( -z_soil(l)/z_nitrogen ) )

    ENDDO ! Loop over # soil layers

    !---TEMP---
    WRITE(numout,*) 't_nitrogen, ', t_nitrogen(:)
    !----------

    !! 1.2.2.2 Convolution for soil moisture
    !  Integrate over # soil layers
    h_nitrogen(:) = zero

    DO l = 1, nbdl ! Loop over # soil layers

       h_nitrogen(:) = &
            h_nitrogen(:) + soilhum_month(:,l) * rpc(:) * &
            ( EXP( -z_soil(l-1)/z_nitrogen ) - EXP( -z_soil(l)/z_nitrogen ) )

    ENDDO ! Loop over # soil layers
    
    !---TEMP---
    WRITE(numout,*) 'h_nitrogen, ', h_nitrogen(:)
    !----------

    !! 1.2.3 Separate between natural and agrigultural LAI
    !  The model distinguishes different natural PFTs but does not contain information
    !  on whether these PFTs are spatially separated or mixed. In line with the DGVM the 
    !  models treats the natural PFT's as mixed. Therefore, the average LAI over the
    !  natural PFTs is calculated to estimate light stress. Agricultural PFTs are spatially
    !  separated. 
    natveg_tot(:) = 0.0
    lai_nat(:) = 0.0

    DO j = 2, nvm ! Loop over # PFTs

       IF ( natural(j) ) THEN

          ! Mask agricultural vegetation
          veget_max_nat(:,j) = veget_max(:,j)

       ELSE

          ! Mask natural vegetation
          veget_max_nat(:,j) = zero

       ENDIF

       ! Sum up fraction of natural space covered by vegetation
       natveg_tot(:) = natveg_tot(:) + veget_max_nat(:,j)

       ! Sum up lai
       lai_nat(:) = lai_nat(:) + veget_max_nat(:,j) * lai(:,j)

    ENDDO ! Loop over # PFTs

    DO j = 2, nvm ! Loop over # PFTs

       IF ( natural(j) ) THEN

          ! Use the mean LAI over all natural PFTs when estimating light stress 
          ! on a specific natural PFT
          lai_around(:,j) = lai_nat(:)

       ELSE

          ! Use the actual LAI (specific for that PFT) when estimating light
          ! stress on a specific agricultural PFT
          lai_around(:,j) = lai(:,j)

       ENDIF

    ENDDO ! Loop over # PFTs


    !! 1.2.4 Calculate LAI threshold below which carbohydrate reserve is used.
    !  Lai_max is a PFT-dependent parameter specified in stomate_constants.f90
    lai_happy_old(:) = lai_max(:) * lai_max_to_happy
    
    
    !! 1.2.5 Set allocatable biomass
    !  Total allocatable biomass during this time step determined from GPP.
    !  GPP was calculated as CO2 assimilation in enerbil.f90
    !  This is ignored later in the code if control%ok_functional_allocation is true
    bm_alloc_tot(:,:) = gpp(:,:) * dt 


 !! 2. Use carbohydrate reserve to support growth and update leaf age

    ! Save old leaf mass, biomass got last updated in stomate_phenology.f90
    lm_old(:,:) = biomass(:,:,ileaf,icarbon)

    DO j = 2, nvm ! Loop over # PFTs 

       !! 2.1 Calculate demand for carbohydrate reserve to support leaf and root growth.
       !  Maximum time (days) since start of the growing season during which carbohydrate 
       !  may be used
       IF ( is_tree(j) ) THEN

          reserve_time = reserve_time_tree

       ELSE
          reserve_time = reserve_time_grass

       ENDIF

       ! Growth is only supported by the use of carbohydrate reserves if the following 
       ! conditions are  statisfied:\n
       ! - PFT is not senescent;\n
       ! - LAI must be low (i.e. below ::lai_happy_old) and\n
       ! - Day of year of the simulation is in the beginning of the growing season.
       WHERE ( ( biomass(:,j,ileaf,icarbon) .GT. zero ) .AND. ( .NOT. senescence(:,j) ) .AND. &
            ( lai(:,j) .LT. lai_happy_old(j) ) .AND. ( when_growthinit(:,j) .LT. reserve_time ) ) 

            ! Determine the mass from the carbohydrate reserve that can be used @tex $(gC m^{-2})$ @endtex. 
            ! Satisfy the demand or use everything that is available 
            ! (i.e. ::biomass(:,j,icarbres,icarbon)). Distribute the demand evenly over the time 
            ! required (::tau_leafinit) to develop a minimal canopy from reserves (::lai_happy_old)
            ! Needs to be additive, since reserves could already have been used in stomate_phenology
            ! old use_reserve(:) = &
            ! old     MIN( biomass(:,j,icarbres,icarbon), &
            ! old     2._r_std * dt/tau_leafinit * lai_happy_old(j)/ sla(j) ).
            use_reserve(:,j) = MIN( biomass(:,j,icarbres,icarbon), &
               & deux * dt / tau_leafinit * lai_happy_old(j) / sla(j) )

!           +++CHECK+++
!$            use_reserve(:,j) = use_reserve(:,j) + MIN( biomass(:,j,icarbres,icarbon) * 0.1, &
!$                 deux * dt / tau_leafinit * lai_happy_old(j) / sla(j) )
!           +++++++++++

            ! Distribute the reserve over leaves and fine roots.
	    ! The part of the reserve going to the leaves is the ratio of default leaf allocation to default root and leaf allocation.
	    ! The remaining of the reserve is alocated to the roots.
            transloc_leaf(:,j) = L0(j)/(L0(j)+R0) * use_reserve(:,j)
            biomass(:,j,ileaf,icarbon) = biomass(:,j,ileaf,icarbon) + transloc_leaf(:,j)
            biomass(:,j,iroot,icarbon) = biomass(:,j,iroot,icarbon) + ( use_reserve(:,j) - transloc_leaf(:,j) )

            ! Adjust the carbohydrate reserve mass by accounting for the reserves allocated to leaves and roots during
            ! this time step
            biomass(:,j,icarbres,icarbon) = biomass(:,j,icarbres,icarbon) - use_reserve(:,j)

       ELSEWHERE

           transloc_leaf(:,j) = zero

       ENDWHERE
      
       !---TEMP---
       WRITE(numout,*) 'use_reserve, ', use_reserve(:,j)
       WRITE(numout,*) 'biomass(icarbres), ', biomass(:,j,icarbres,icarbon)+use_reserve(:,j)
       WRITE(numout,*) 'other part, ', deux * dt / tau_leafinit * lai_happy_old(j) / sla(j)
       !----------
       
 
 !! 3. Update leaf age

       !  Leaf age is needed for calculation of turnover and vmax in stomate_turnover.f90 and stomate_vmax.f90 routines. 
       !  Leaf biomass is distributed according to its age into several "age classes" with age class=1 representing the
       !  youngest class, and consisting of the most newly allocated leaf biomass 
    
       !! 3.1 Update quantity and age of the leaf biomass in the youngest class
       !  The new amount of leaf biomass in the youngest age class (leaf_mass_young) is the sum of :
       !  - the leaf biomass that was already in the youngest age class (leaf_frac(:,j,1) * lm_old(:,j)) with the 
       !    leaf age given in leaf_age(:,j,1) 
       !  - and the new biomass allocated to leaves (bm_alloc(:,j,ileaf,icarbon)) with a leaf age of zero.
       leaf_mass_young(:,j) = leaf_frac(:,j,1) * lm_old(:,j) + transloc_leaf(:,j)

       ! The age of the updated youngest age class is the average of the ages of its 2 components: bm_alloc(leaf) of age
       ! '0', and leaf_frac*lm_old(=leaf_mass_young-bm_alloc) of age 'leaf_age(:,:,1)' 
       WHERE ( ( transloc_leaf(:,j) .GT. min_stomate ) .AND. ( leaf_mass_young(:,j) .GT. min_stomate ) )
          
          leaf_age(:,j,1) = MAX ( zero, leaf_age(:,j,1) * ( leaf_mass_young(:,j) - transloc_leaf(:,j) ) / &
               & leaf_mass_young(:,j) )
          
       ENDWHERE


       !! 3.2 Update leaf fractions
       !      Update fractions of leaf biomass in each age class (fraction in youngest class increases)

       !! 3.2.1 Update age of youngest leaves
       !        For age class 1 (youngest class), because we have added biomass to the youngest class, we need to update
       !        the fraction of total leaf biomass that belongs to the youngest age class : updated mass in class divided
       !        by new total leaf mass
       WHERE ( biomass(:,j,ileaf,icarbon) .GT. min_stomate )

          leaf_frac(:,j,1) = leaf_mass_young(:,j) / biomass(:,j,ileaf,icarbon)

       ENDWHERE


       !! 3.2.2 Update age of other age classes
       !        Because the total leaf biomass has changed, we need to update the fraction of leaves in each age class:
       !        mass in leaf age class (from previous fraction of leaves in this class and previous total leaf biomass) 
       !        divided by new total mass
       DO m = 2, nleafages	! Loop over # leaf age classes

          WHERE ( biomass(:,j,ileaf,icarbon) .GT. min_stomate )

             leaf_frac(:,j,m) = leaf_frac(:,j,m) * lm_old(:,j) / biomass(:,j,ileaf,icarbon)

          ENDWHERE

       ENDDO	! Loop over # leaf age classes

    ENDDO ! loop over # PFTs


 !! 4. Calculate allocatable fractions of biomass production (NPP) 
     
    ! Calculate fractions of biomass production (NPP) to be allocated to the different 
    ! biomass components.\n
    ! The fractions of NPP allocated (0-1, unitless) to the different compartments depend on the
    ! availability of light, water, and nitrogen.
    DO j = 2, nvm ! Loop over # PFTs

       ! Reset values
       RtoLSR(:) = zero
       LtoLSR(:) = zero
       StoLSR(:) = zero
        
       !! 4.1 Age dependency of aboveground allocation
       !  For trees, partitioning between above and belowground sapwood biomass is a function 
       !  of age. An older tree gets more allocation to the aboveground sapwoood than a younger tree.
       !  For the other PFTs it is prescribed. 
       !  ::alloc_min, ::alloc_max and ::demi_alloc are specified in stomate_constants.f90
       IF ( is_tree(j) ) THEN

          alloc_sap_above (:) = alloc_min(j) + (alloc_max(j) - alloc_min(j)) * &
             &   ( un - EXP( -age(:,j) / demi_alloc(j) ) )

       ELSE

          alloc_sap_above(:) = alloc_sap_above_grass

       ENDIF  ! tree

       !---TEMP---
       WRITE(numout,*) 'age, ', age(:,j)
       !----------

       !! 4.2 Calculate light stress, water stress and proxy for nitrogen stress.
       !  For the limiting factors a low value indicates a strong limitation
       WHERE ( biomass(:,j,ileaf,icarbon) .GT. min_stomate )

          !! 4.2.1 Light stress
          !  Light stress is a function of the mean lai on the natural part of the grid box 
          !  and of the PFT-specific LAI for agricultural crops. In line with the DGVM, natural
          !  PFTs in the same gridbox are treated as if they were spatially mixed whereas 
          !  agricultural PFTs are considered to be spatially separated.
          !  The calculation of the lights stress depends on the extinction coefficient (set to 0.5)
          !  and of a mean LAI.
          WHERE( lai_around(:,j) < max_possible_lai )
             
             limit_L(:) = MAX( 0.1_r_std, EXP( -ext_coeff(j) * lai_around(:,j) ) )

          ELSEWHERE
             
             limit_L(:) = 0.1_r_std
          
          ENDWHERE

 
          !! 4.2.2 Water stress
          !  Water stress is calculated as the weekly moisture availability.
          !  Weekly moisture availability is calculated in stomate_season.f90.
          limit_W(:) = MAX( 0.1_r_std, MIN( un, moiavail_week(:,j) ) )


          !! 4.2.3 Proxy for nitrogen stress
          !  The proxy for nitrogen stress depends on monthly soil water availability
          !  (::soilhum_month) and monthly soil temperature (::tsoil_month). See section
          !  1.2.2 for details on how ::t_nitrogen and ::h_nitrogen were calculated.\n
          !  Currently nitrogen-stress is calculated for both natural and agricultural PFTs.
          !  Due to intense fertilization of agricultural PFTs this is a strong 
          !  assumption for several agricultural regions in the world (US, Europe, India, ...)
          !  Water stress on nitrogen mineralisation 
          limit_N_hum(:) = MAX( undemi, MIN( un, h_nitrogen(:) ) )

          ! Temperature stress on nitrogen mineralisation using a Q10 decomposition model
          ! where Q10 was set to 2
          limit_N_temp(:) = deux**( ( t_nitrogen(:) - ZeroCelsius - Nlim_tref ) / Nlim_Q10 )
          limit_N_temp(:) = MAX( 0.1_r_std, MIN( un, limit_N_temp(:) ) )

          ! Combine water and temperature factors to get total nitrogen stress
          limit_N(:) = MAX( 0.1_r_std, MIN( un, limit_N_hum(:) * limit_N_temp(:) ) )

          ! Take the most limiting factor among soil water and nitrogen 
          limit_WorN(:) = MIN( limit_W(:), limit_N(:) )

          ! Take the most limiting factor among aboveground (i.e. light) and belowground 
          ! (i.e. water & nitrogen) limitations
          limit(:) = MIN( limit_WorN(:), limit_L(:) )


          !! 4.3 Calculate ratio between allocation to leaves, sapwood and roots
          !  Partitioning between belowground and aboveground biomass components is assumed 
          !  to be proportional to the ratio of belowground and aboveground stresses.\n
          !  \latexonly 
          !  \input{alloc1.tex} 
          !  \endlatexonly
          !  Root allocation is the default root allocation corrected by a normalized ratio of aboveground 
          !  stress to total stress. The minimum root allocation is 0.15.
          RtoLSR(:) = MAX( .15_r_std, R0 * trois * limit_L(:) / ( limit_L(:) + deux * limit_WorN(:) ) )

          ! Sapwood allocation is the default sapwood allocation corrected by a normalized ratio of 
          ! belowground stress to total stress.
          StoLSR(:) = S0(j) * trois * limit_WorN(:) / ( deux * limit_L(:) + limit_WorN(:) )

          ! Leaf allocation is calculated as the remaining allocation fraction
	  ! The range of variation of leaf allocation is constrained by ::min_LtoLSR and ::max_LtoLSR. 
          LtoLSR(:) = un - RtoLSR(:) - StoLSR(:)
          LtoLSR(:) = MAX( min_LtoLSR, MIN( max_LtoLSR, LtoLSR(:) ) )

          ! Roots allocation is recalculated as the residual carbon after leaf allocation has been calculated. 
          RtoLSR(:) = un - LtoLSR(:) - StoLSR(:)

       ENDWHERE
	   
       ! Check whether allocation needs to be adjusted. If LAI exceeds maximum LAI 
       ! (::lai_max), no addition carbon should be allocated to leaf biomass. Allocation is 
       ! then partioned between root and sapwood biomass.
       WHERE ( (biomass(:,j,ileaf,icarbon) .GT. min_stomate) .AND. (lai(:,j) .GT. lai_max(j)) )

          StoLSR(:) = StoLSR(:) + LtoLSR(:)
          LtoLSR(:) = zero

       ENDWHERE
       

       !! 4.4 Calculate the allocation fractions.
       !  The allocation fractions (::f_alloc) are an output variable (0-1, unitless). f_alloc
       !  has three dimensions (npts,nvm,nparts). Where ::npts is the number of grid cells, ::nvm is the 
       !  number of PFTs and ::nparts the number of biomass components. Currently six biomass compartments
       !  are distinguished: (1) Carbon reserves, (2) Aboveground sapwood, (3) Belowground 
       !  sapwood, (4) Roots, (5) fruits/seeds and (6) Leaves.@tex $(gC m^{-2})$ @endtex \n
       DO i = 1, npts ! Loop over grid cells 

          IF ( biomass(i,j,ileaf,icarbon) .GT. min_stomate ) THEN
      
            IF ( senescence(i,j) ) THEN
                
               !! 4.4.1 Allocate all C to carbohydrate reserve
               !  If the PFT is senescent allocate all C to carbohydrate reserve, 
               !  then the allocation fraction to reserves is 1. 
               f_alloc(i,j,icarbres) = un

            ELSE

               !! 4.4.2 Allocation during the growing season  
               f_alloc(i,j,ifruit) = f_fruit

               ! Allocation to the carbohydrate reserve is proportional to leaf and root 
               ! allocation. If carbon is allocated to the carbohydrate reserve, rescaling 
               ! of allocation factors is required to ensure carbon mass preservation.
               ! Carbon is allocated to the carbohydrate reserve when the pool size of the 
               ! reserve is less than the carbon needed to grow a canopy twice the size of 
               ! the maximum LAI (::lai_max). Twice the size was used as a threshold because
               ! the reserves needs to be sufficiently to grow a canopy and roots. In case 
               ! the carbohydrate pool is full, there is no need to rescale the other
               ! allocation factors.
	       ! If there is no rescaling of the allocation factors (carbres=1, no carbon put
	       ! to reserve), then fraction remaining after fruit allocation (1-fruit_alloc) 
	       ! is distributed between leaf, root and sap (sap carbon also distributed between   
	       ! sap_above and sap_below with factor alloc_sap_above).
	       ! If carbon is allocated to the carbohydrate reserve, all these factors are 
	       ! rescaled through carb_rescale, and an allocation fraction for carbohydrate pool 
	       ! appears. carb_rescale depends on the parameter (::ecureuil). 
               ! (::ecureuil) is the fraction of primary leaf and root allocation put into 
               ! reserve, it is specified in stomate_constants.f90 and is either 0 or 1.
               IF ( ( biomass(i,j,icarbres,icarbon) * sla(j) ) .LT. deux * lai_max(j) ) THEN

                    carb_rescale(i) = un / ( un + ecureuil(j) * ( LtoLSR(i) + RtoLSR(i) ) )
               
               ELSE
                    carb_rescale(i) = un
               
               ENDIF

               f_alloc(i,j,ileaf) = LtoLSR(i) * ( un - f_alloc(i,j,ifruit) ) * carb_rescale(i)
               f_alloc(i,j,isapabove) = StoLSR(i) * alloc_sap_above(i) * &
                  ( un - f_alloc(i,j,ifruit) ) * carb_rescale(i)
               f_alloc(i,j,isapbelow) = StoLSR(i) * ( un - alloc_sap_above(i) ) * &
                  ( un - f_alloc(i,j,ifruit) ) * carb_rescale(i)
               f_alloc(i,j,iroot) = RtoLSR(i) * ( un - f_alloc(i,j,ifruit) ) * carb_rescale(i)
               f_alloc(i,j,icarbres) = ( un - carb_rescale(i) ) * ( un - f_alloc(i,j,ifruit) )

               ENDIF  ! Is senescent?

            ENDIF  ! There are leaves

         ENDDO  ! Loop over # pixels - domain size

      ENDDO  ! loop over # PFTs


 !! 5. Calculate maintenance and growth respiration

    ! First, total maintenance respiration for the whole plant is calculated by summing maintenance 
    ! respiration of the different plant compartments. Then, maintenance respiration is subtracted 
    ! from whole-plant allocatable biomass (up to a maximum fraction of the total allocatable biomass). 
    ! Growth respiration is then calculated for each plant compartment as a fraction of remaining 
    ! allocatable biomass for this compartment. NPP is calculated by substracting total autotrophic 
    ! respiration from GPP i.e. NPP = GPP - maintenance resp - growth resp.
    DO j = 2,nvm	! Loop over # of PFTs       

       !! 5.1 Maintenance respiration of the different plant parts 
       !  Maintenance respiration of the different plant parts is calculated in 
       !  stomate_resp.f90 as a function of the plant's temperature, 
       !  the long term temperature and plant coefficients 
       resp_maint(:,j) = zero

       !  Following the calculation of hourly maintenance respiration, verify that 
       !  the PFT has not been killed after calcul of resp_maint_part in stomate.
       DO k = 1, nparts

          WHERE (PFTpresent(:,j))

             resp_maint(:,j) = resp_maint(:,j) + resp_maint_part(:,j,k)

          ENDWHERE

       ENDDO
       
       !! 5.2 Substract maintenance respiration from allocatable biomass
       !  The total maintenance respiration calculated in 2.2 is substracted from the newly 
       !  produced allocatable biomass (bm_alloc_tot). However, ensure that not all allocatable 
       !  biomass is removed by setting a maximum to the fraction of allocatable biomass used 
       !  for maintenance respiration: tax_max. If the maintenance respiration is larger than 
       !  tax_max,the amount tax_max is taken from allocatable biomass, and the remaining of 
       !  maintenance respiration is taken from the tissues themselves (biomass). We suppose 
       !  that respiration is not dependent on leaf age -> therefore the leaf age structure is 
       !  not changed. 
       !  The maximum fraction of allocatable biomass used for respiration is defined as tax_max. 
       !  The value of tax_max is set in the declarations section (0.4 Local variables) of this
       !  routine

       !! 5.2.1 Resource based allocatable biomass   
       !  maximum part of allocatable biomass used for respiration
       bm_tax_max(:) = tax_max * bm_alloc_tot(:,j)

       ! If there is enough allocatable biomass to cover maintenance respiration, 
       ! then biomass associated with maintenance respiration is removed from allocatable biomass
       WHERE ( ( bm_alloc_tot(:,j) .GT. zero ) .AND. &
            ( ( resp_maint(:,j) * dt ) .LT. bm_tax_max(:) ) )	
	  
          bm_alloc_tot(:,j) = bm_alloc_tot(:,j) - resp_maint(:,j) * dt

       ELSEWHERE ( resp_maint(:,j) .GT. min_stomate )

	  ! If there is not enough allocatable biomass to cover maintenance respiration, the  
          ! - maximum allowed allocatable biomass (bm_tax_max) is removed from allocatable biomass.   
          bm_alloc_tot(:,j) = bm_alloc_tot(:,j) - bm_tax_max(:)

          ! ::bm_pump is the amount of maintenance respiration that exceeds the maximum allocatable biomass
	  ! This amount of biomass still needs to be respired and will be removed from tissues biomass of each
	  ! plant compartment
          bm_pump(:) = resp_maint(:,j) * dt - bm_tax_max(:)

          ! The biomass is removed from each plant compartment tissues as the ratio of the maintenance		
          ! respiration of the plant compartment to the total maintenance respiration (resp_maint_part/resp_maint)
          biomass(:,j,ileaf,icarbon) = biomass(:,j,ileaf,icarbon) - &
             bm_pump(:) * resp_maint_part(:,j,ileaf) / resp_maint(:,j)
          biomass(:,j,isapabove,icarbon) = biomass(:,j,isapabove,icarbon) - &
             bm_pump(:) * resp_maint_part(:,j,isapabove) / resp_maint(:,j)
          biomass(:,j,isapbelow,icarbon) = biomass(:,j,isapbelow,icarbon) - &
             bm_pump(:) * resp_maint_part(:,j,isapbelow) / resp_maint(:,j)
          biomass(:,j,iroot,icarbon) = biomass(:,j,iroot,icarbon) - &
             bm_pump(:) * resp_maint_part(:,j,iroot) / resp_maint(:,j)
          biomass(:,j,ifruit,icarbon) = biomass(:,j,ifruit,icarbon) - &
             bm_pump(:) * resp_maint_part(:,j,ifruit) / resp_maint(:,j)
          biomass(:,j,icarbres,icarbon) = biomass(:,j,icarbres,icarbon) - &
             bm_pump(:) * resp_maint_part(:,j,icarbres) / resp_maint(:,j)
          ! Resource based allocation does not uses a labile carbon pool
          biomass(:,j,ilabile,icarbon) = zero

       ENDWHERE  ! there is enough allocatable biomass to cover maintenance respiration

     
       !! 5.3 Allocate allocatable biomass to different plant compartments.
       !      The amount of allocatable biomass to each compartment is a fraction ::f_alloc of the total 
       !      allocatable biomass. For the resource-based allocation ::f_alloc of the different plant parts 
       !      is calculated in stomate_alloc.f90
       DO k = 1, nparts

          bm_alloc(:,j,k,icarbon) = f_alloc(:,j,k) * bm_alloc_tot(:,j)

       ENDDO
     

       !! 5.4 Calculate growth respiration of each plant compartment
       !      Growth respiration of a plant compartment is a fraction of the allocatable biomass remaining after
       !      maintenance respiration losses have been taken into account. The fraction of allocatable biomass 
       !      removed for growth respiration is the same for all plant compartments and is defined by the parameter
       !      frac_growth_resp in stomate_constants.f90. Allocatable biomass ::bm_alloc is updated as a result of 
       !      the removal of growth resp (gC m-2 dt-1).
       DO k = 1,nparts

          resp_growth_part(:,k) = frac_growthresp(j) * bm_alloc(:,j,k,icarbon) / dt
          bm_alloc(:,j,k,icarbon) =  ( 1. - frac_growthresp(j) ) * bm_alloc(:,j,k,icarbon)

       ENDDO
      

       !! 5.5 Total growth respiration 
       !      Calculate total growth respiration of the plant as the sum of growth respiration of all plant parts	
       resp_growth(:,j) = zero
       DO k = 1, nparts
          
          ! Total growth respiration. Calculate total growth respiration of the plant as the 
          ! sum of growth respiration of all plant parts
          resp_growth(:,j) = resp_growth(:,j) + resp_growth_part(:,k)

       ENDDO ! # biomass compartments

    ENDDO ! # of PFTs

    
 !! 6. Update the biomass with newly allocated biomass after respiration
 
    !  Save the old leaf biomass for later. "old" leaf mass is leaf mass after maintenance respiration in the case 
    !  where maintenance respiration has required taking biomass from tissues in section 3.2    
    lm_old(:,:) = biomass(:,:,ileaf,icarbon)
    biomass(:,:,:,icarbon) = biomass(:,:,:,icarbon) + bm_alloc(:,:,:,icarbon)

   
 !! 7. Deal with negative biomasses

    !  Biomass can become negative in some rare cases, as the GPP can be negative (dark respiration). In this case, 
    !  we set biomass to some small value min_stomate. This creation of matter (carbon) is taken into account by 
    !  decreasing the autotrophic respiration by the same amount that has been added to biomass for it to become 
    !  positive. In this case, maintenance respiration can become negative !!!
   
    DO k = 1, nparts	! Loop over # of plant parts

       WHERE ( biomass(:,:,k,icarbon) .LT. zero )

          bm_create(:,:) = min_stomate - biomass(:,:,k,icarbon)     
          biomass(:,:,k,icarbon) = biomass(:,:,k,icarbon) + bm_create(:,:)    
          resp_maint(:,:) = resp_maint(:,:) - bm_create(:,:) / dt

       ENDWHERE

    ENDDO	! Loop over # plant parts
    

 !! 8. Calculate NPP (See Eq 1 in header)
    
    ! Calculate the NPP @tex $(gC.m^{-2}dt^{-1})$ @endtex as the difference between GPP
    ! and autotrophic respiration (maintenance and growth respirations)
    ! This is a diagnostic calculation
    npp(:,:) = gpp(:,:) - resp_growth(:,:) - resp_maint(:,:)
    

 !! 9. Update leaf age

    !  Leaf age is needed for calculation of turnover and vmax in stomate_turnover.f90 and stomate_vmax.f90 routines. 
    !  Leaf biomass is distributed according to its age into several "age classes" with age class=1 representing the
    !  youngest class, and consisting of the most newly allocated leaf biomass 
    
    !! 9.1 Update quantity and age of the leaf biomass in the youngest class
    !      The new amount of leaf biomass in the youngest age class (leaf_mass_young) is the sum of :
    !      - the leaf biomass that was already in the youngest age class (leaf_frac(:,j,1) * lm_old(:,j)) with the 
    !        leaf age given in leaf_age(:,j,1) 
    !      - and the new biomass allocated to leaves (bm_alloc(:,j,ileaf)) with a leaf age of zero.
    DO j = 2,nvm

       leaf_mass_young(:,j) = leaf_frac(:,j,1) * lm_old(:,j) + bm_alloc(:,j,ileaf,icarbon)

    ENDDO

    ! The age of the updated youngest age class is the average of the ages of its 2 components: bm_alloc(leaf) of age
    ! '0', and leaf_frac*lm_old(=leaf_mass_young-bm_alloc) of age 'leaf_age(:,j,1)' 
    DO j = 2,nvm

       WHERE ( ( bm_alloc(:,j,ileaf,icarbon) .GT. zero ) .AND. ( leaf_mass_young(:,j) .GT. zero ) )

          leaf_age(:,j,1) = MAX( zero, leaf_age(:,j,1) * ( leaf_mass_young(:,j) - bm_alloc(:,j,ileaf,icarbon) ) / &
             & leaf_mass_young(:,j) )
          
       ENDWHERE

    ENDDO

    !  For age class 1 (youngest class), because we have added biomass to the youngest class, we need to update
    !  the fraction of total leaf biomass that belongs to the youngest age class : updated mass in class divided
    !  by new total leaf mass
    DO j = 2,nvm

       WHERE ( biomass(:,j,ileaf,icarbon) .GT. min_stomate )

          leaf_frac(:,j,1) = leaf_mass_young(:,j) / biomass(:,j,ileaf,icarbon)

       ENDWHERE

    ENDDO

    !! 9.2 Update age of other age classes
    !  Because the total leaf biomass has changed, we need to update the fraction of leaves in each age class:
    !  mass in leaf age class (from previous fraction of leaves in this class and previous total leaf biomass) 
    !  divided by new total mass
    DO m = 2, nleafages	

       DO j = 2,nvm

          WHERE ( biomass(:,j,ileaf,icarbon) .GT. min_stomate )

             leaf_frac(:,j,m) = leaf_frac(:,j,m) * lm_old(:,j) / biomass(:,j,ileaf,icarbon)

          ENDWHERE

       ENDDO

    ENDDO


 !! 10. Update whole-plant age 
    
    !! 10.1 PFT age
    !  At every time step, increase age of the biomass that was already present at previous time step. 
    !  Age is expressed in years, and the time step 'dt' in days so age increase is: dt divided by number 
    !  of days in a year.
    WHERE ( PFTpresent(:,:) )

       age(:,:) = age(:,:) + dt/one_year

    ELSEWHERE

       age(:,:) = zero

    ENDWHERE


    !! 10.2 Age of grasses and crops
    !  For grasses and crops, biomass with age 0 has been added to the whole plant with age 'age'. New biomass is the sum of 
    !  the current total biomass in all plant parts (bm_new), bm_new(:) = SUM( biomass(:,j,:,icarbon), DIM=2 ). The biomass 
    !  that has just been added is the sum of the allocatable biomass of all plant parts (bm_add), its age is zero. bm_add(:) = 
    !  SUM( bm_alloc(:,j,:,icarbon), DIM=2 ). Before allocation, the plant biomass is bm_new-bm_add, its age is "age(:,j)". 
    !  The age of the new biomass is the average of the ages of previous and added biomass.
    !  For trees, age is treated in "establish" if vegetation is dynamic, and in turnover routines if it is static (in this 
    !  case, only the age of the heartwood is accounted for).
    DO j = 2,nvm

       IF ( .NOT. is_tree(j) ) THEN

          bm_new(:) = biomass(:,j,ileaf,icarbon) + biomass(:,j,isapabove,icarbon) + &
               biomass(:,j,iroot,icarbon) + biomass(:,j,ifruit,icarbon)
          bm_add(:) = bm_alloc(:,j,ileaf,icarbon) + bm_alloc(:,j,isapabove,icarbon) + &
               bm_alloc(:,j,iroot,icarbon) + bm_alloc(:,j,ifruit,icarbon)

          WHERE ( ( bm_new(:) .GT. zero ) .AND. ( bm_add(:) .GT. min_stomate ) )

             age(:,j) = age(:,j) * ( bm_new(:) - bm_add(:) ) / bm_new(:)
          
          ENDWHERE

       ENDIF ! is .NOT. tree

    ENDDO  ! Loop over #PFTs 


 !! 11. Write history files

    ! Save in history file the variables describing the biomass allocated to the plant parts
    CALL histwrite (hist_id_stomate, 'BM_ALLOC_LEAF', itime, &
         bm_alloc(:,:,ileaf,icarbon), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'BM_ALLOC_SAP_AB', itime, &
         bm_alloc(:,:,isapabove,icarbon), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'BM_ALLOC_SAP_BE', itime, &
         bm_alloc(:,:,isapbelow,icarbon), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'BM_ALLOC_ROOT', itime, &
         bm_alloc(:,:,iroot,icarbon), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'BM_ALLOC_FRUIT', itime, &
         bm_alloc(:,:,ifruit,icarbon), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'BM_ALLOC_RES', itime, &
         bm_alloc(:,:,icarbres,icarbon), npts*nvm, horipft_index)

    IF (bavard.GE.4) WRITE(numout,*) 'Leaving resource limited growth'


  END SUBROUTINE growth_res_lim

END MODULE stomate_growth_res_lim
